import re

from .mtgpics_overrides import (
    set_override,
    cn_set_override,
    override_pic,
    cn_override,
)

from .cache_url import get


def find_mtgpic_link(set_name: str, cn: int, backside: bool) -> str:
    set_name, cn = (
        cn_set_override[(set_name, cn)]
        if (set_name, cn) in cn_set_override
        else (set_name, cn)
    )
    set_name = set_override[set_name] if set_name in set_override else set_name
    cn = (
        cn_override[set_name][cn]
        if set_name in cn_override and cn in cn_override[set_name]
        else cn
    )
    mtgpic_r = get(f"https://www.mtgpics.com/card?ref={set_name}{cn:03d}")
    content = mtgpic_r.content.decode("iso-8859-1").splitlines()
    illus_id = ""
    for c in content:
        res = re.findall(r"""onclick="LoadIllus..([0-9]*)""", c)
        if res:
            if len(res) > 1 and backside:
                illus_id = res[1]
            else:
                illus_id = res[0]
    if illus_id == "":
        raise RuntimeError()

    illuspage = get(f"https://www.mtgpics.com/load_illus?i={illus_id}")
    content = illuspage.content.decode("iso-8859-1").splitlines()
    for c in content:
        res = re.match(r"<img src=([^>]*)>", c)
        if res is not None:
            pic = (
                override_pic[res.group(1)]
                if res.group(1) in override_pic
                else res.group(1)
            )
            return f"https://www.mtgpics.com/{pic}"
    return ""


def find_artwork(
    set_name: str, cn_: str, backup_url: str, backside: bool = False
) -> str:
    cn = int("".join(c for c in cn_ if c in "0123456789"))
    # crop = False
    try:
        url = find_mtgpic_link(set_name, cn, backside)
        # crop = True
        # pics_result = get(url)
    except RuntimeError:
        print(f"Scryfall backup for {set_name} {cn}")
        url = backup_url
        # pics_result = get(backup_url)
        # crop = False

    # img = Image.open(BytesIO(pics_result.content))

    # if crop:
    #     # cut off bottom ~6 percent
    #     height = img.height
    #     img = img.crop((0, 0, img.width, int(height * 0.94)))
    return url
