import sqlite3
from typing import Optional
import requests
import json

from requests.models import Response


class Cache:
    def __init__(self, sqlite_file: str, threading: bool = False):
        self.sqlite_file = sqlite_file
        self.threading = threading
        self.init = False
        if not threading:
            self.conn = sqlite3.connect(sqlite_file)
            self.conn.execute(
                "CREATE TABLE IF NOT EXISTS cache (url TEXT PRIMARY KEY, headers TEXT, status_code INTEGER, content BLOB)"
            )
            self.init = True

    def get(self, url) -> Response:
        if self.threading:
            conn = sqlite3.connect(self.sqlite_file)
        else:
            conn = self.conn
        cursor = conn.cursor()
        cursor.execute(
            "SELECT headers, status_code, content FROM cache WHERE url = ?", (url,)
        )

        row = cursor.fetchone()
        cursor.close()
        if row is None:
            response = requests.get(url)
            headers = json.dumps(dict(response.headers))
            content = response.content
            status_code = response.status_code

            conn.execute(
                "INSERT INTO cache (url, headers, status_code, content) VALUES (?, ?, ?, ?)",
                (url, headers, status_code, content),
            )
            conn.commit()
        else:
            response = Response()
            response._content = row[2]
            response.status_code = row[1]
            response.headers = json.loads(row[0])
        return response


__cache__ = None


def get_cache(sqlite_file: Optional[str] = None, threading: bool = False) -> Cache:
    global __cache__
    if __cache__ is None:
        if sqlite_file is None:
            sqlite_file = ":memory:"
        __cache__ = Cache(sqlite_file, threading)
    return __cache__


def get(url: str) -> Response:
    return get_cache().get(url)
