from __future__ import annotations

from typing import Any, Literal, Optional, TypeAlias
from dataclasses import dataclass, field

from PIL import Image


@dataclass
class ArtInfo:
    url: Optional[str]
    # anchor: Literal["top", "left", "top-left"] = "top"
    x_perc_offset: Optional[float] = None
    y_perc_offset: Optional[float] = None
    zoom: Optional[float] = None
    image: Optional[Image.Image] = None

    def merge(self, other: "ArtInfo"):
        if self.url is None:
            self.url = other.url
        if self.x_perc_offset is None:
            self.x_perc_offset = other.x_perc_offset
        if self.y_perc_offset is None:
            self.y_perc_offset = other.y_perc_offset
        if self.zoom is None:
            self.zoom = other.zoom
        if self.image is None:
            self.image = other.image


@dataclass
class Face:
    name: str
    mana_cost: str
    colors: list[str]
    type_line: str
    supertypes: list[str]
    types: list[str]
    subtypes: list[str]
    oracle_text: list[str]
    flavor_text: Optional[str]
    img: ArtInfo
    artist: str
    border_color: str
    produces: Optional[str]
    power: Optional[str]
    toughness: Optional[str]
    loyalty: Optional[str]
    color_indicator: Optional[str]
    layout: Optional[str]
    textless: bool


@dataclass
class Card:
    rarity: str
    set: str
    cn: str
    cn_int: int
    cn_add: str
    layout: str
    faces: list[Face]


BBox: TypeAlias = tuple[int, int, int, int]


@dataclass
class Frame:
    images: dict[str, str]
    art_box: Optional[BBox] = field(default=None)
    title_box: Optional[BBox] = field(default=None)
    mana_box: Optional[BBox] = field(default=None)
    type_box: Optional[BBox] = field(default=None)
    rules_box: Optional[BBox] = field(default=None)
    pt_box: Optional[BBox] = field(default=None)
    set_symbol_box: Optional[BBox] = field(default=None)
    artist_box: Optional[BBox] = field(default=None)
    copyright_box: Optional[BBox] = field(default=None)


@dataclass
class Dimensions:
    art_box: BBox
    title_box: BBox
    title_left_box: BBox
    title_right_box: BBox
    mana_box: BBox
    type_box: BBox
    rules_boxes: list[BBox]
    pt_box: BBox
    set_symbol_box: BBox
    artist_box: BBox
    copyright_box: BBox
    mdfc_box: BBox
    title_box2: BBox
    mana_box2: BBox
    type_box2: BBox
    proxy_box: BBox

    @classmethod
    def load(cls, data: dict[str, Any]) -> Dimensions:
        dimensions: dict[str, Any] = {
            entry: tuple(box) if isinstance(box, list) else list(map(tuple, box))
            for entry, box in data.items()
        }
        return cls(**dimensions)
