from PIL import Image, ImageEnhance
import sys
import math


def mergepixel(
    p1: tuple[int, int, int, int], p2: tuple[int, int, int, int]
) -> tuple[int, int, int, int]:
    new_alpha = p1[3] + p2[3] * (1 - p1[3])
    new_r = (p1[0] * p1[3] + p2[0] * p2[3] * (1 - p1[3])) // new_alpha
    new_g = (p1[1] * p1[3] + p2[1] * p2[3] * (1 - p1[3])) // new_alpha
    new_b = (p1[2] * p1[3] + p2[2] * p2[3] * (1 - p1[3])) // new_alpha
    return (new_r, new_g, new_b, new_alpha)


def quad(x: float) -> float:
    return x


def set_transp1(img: Image.Image) -> Image.Image:
    new_img = Image.new("RGBA", (img.width, img.height))
    for x in range(img.width):
        alpha = 255 - int(quad(x / img.width) * 255)
        for y in range(img.height):
            r, g, b, a = img.getpixel((x, y))
            new_alpha = int((a * alpha) / (255))
            new_img.putpixel((x, y), (r, g, b, new_alpha))
    return new_img


def set_transp(img: Image.Image) -> Image.Image:
    new_img = Image.new("RGBA", (img.width, img.height))
    for x in range(img.width):
        alpha = int(quad(x / img.width) * 256)
        for y in range(img.height):
            r, g, b, a = img.getpixel((x, y))
            new_alpha = int((a * alpha) / (256))
            new_img.putpixel((x, y), (r, g, b, new_alpha))
    return new_img


def fix_alpha(img: Image.Image) -> Image.Image:
    new_img = Image.new("RGBA", (img.width, img.height))
    for x in range(img.width):
        for y in range(img.height):
            r, g, b, a = img.getpixel((x, y))
            if a != 0:
                a = 255
            new_img.putpixel((x, y), (r, g, b, a))
    return new_img


def f(x: float) -> float:
    e = math.e
    k = 17
    x = x - 0.5

    return (((e ** (k * x) - e ** (-k * x)) / (e ** (k * x) + e ** (-k * x))) / 2) + 0.5


# def merge2(left: Image.Image, right: Image.Image) -> Image.Image:
#     width = left.width
#     height = left.height
#     img = Image.new("RGBA", (width, height), (0, 0, 0, 0))
#
#     for x in range(width):
#         r_val = int(f(x / width) * 255)
#         l_val = 255 - r_val
#         for y in range(height):
#             pixel_left = left.getpixel((x, y))
#             pixel_right = right.getpixel((x, y))
#             r = (pixel_left[0] * l_val + pixel_right[0] * r_val) // 255
#             g = (pixel_left[1] * l_val + pixel_right[1] * r_val) // 255
#             b = (pixel_left[2] * l_val + pixel_right[2] * r_val) // 255
#             img.putpixel((x, y), (r, g, b, pixel_left[3]))
#
#     return img


def merge(left: Image.Image, right: Image.Image) -> Image.Image:
    width = left.width
    height = left.height
    mask = Image.new("L", (width, height), (0,))
    for x in range(width):
        val = 255 - int(f(x / width) * 255)
        for y in range(height):
            mask.putpixel((x, y), (val))

    return Image.composite(left, right, mask)


# def overlay2(first: Image.Image, second: Image.Image, perc: float = 0.5) -> Image.Image:
#     width = first.width
#     height = first.height
#     img = Image.new("RGBA", (width, height), (0, 0, 0, 0))
#     img2 = Image.new("RGBA", (width, height), (0, 0, 0, 0))
#
#     img.paste(first)
#     img2.paste(second)
#
#     a = img2.split()[3]
#     alpha = ImageEnhance.Brightness(a).enhance(perc)
#     img2.putalpha(alpha)
#     img.alpha_composite(img2)
#     return img


def overlay(first: Image.Image, second: Image.Image, perc: float = 0.5) -> Image.Image:
    return Image.blend(first, second, perc)


def main() -> None:
    left = Image.open(sys.argv[2])
    # new_left = set_transp(left)
    right = Image.open(sys.argv[1])
    # new_right = set_transp1(right)
    # new_right.alpha_composite(new_left)
    # i = fix_alpha(new_right)
    i = merge(left, right)
    i.save(sys.argv[3])


if __name__ == "__main__":
    main()
